#include "prjlibs-include/standards.h"
#include <sys/time.h>
#include <unistd.h>
#include <errno.h>
#include <limits.h>
#include <signal.h>

#include <skalibs/stddjb.h>
#include "prjlibs-include/constants.h"
#include "prjlibs-include/diewarn.h"
#include "runwhen.h"

char const* PROG="rw-sleep";

static void alarm_handler(int sig) { (void)sig; }

int main(int argc, char** argv) {
  struct itimerval timer;
  struct sigaction action;
  sigset_t alarmset;
  sigset_t blockset;
  int flag_sleep;

  if (argc<2) DIE_USAGE(" {stamp|-} program [arg...]");

  /* set up sigset containing SIGALRM */
  if (sigemptyset(&alarmset)!=0) DIE0(sigemptyset);
  if (sigaddset(&alarmset, SIGALRM)!=0) DIE0(sigaddset);

  /* block SIGALRM ASAP */
  if (sigprocmask(SIG_BLOCK, &alarmset, &blockset)!=0) DIE0(sigprocmask);

  /* figure out how long to sleep */
  if (argv[1][0]=='-' && argv[1][1]=='\0') {
    flag_sleep=1;
    timer.it_value.tv_sec=0;
    timer.it_value.tv_usec=0;
  } else {
    tain now, wake_time;
    tain_now(&now);
    rw_scan(&wake_time, argv[1]);
    if (tain_less(&wake_time, &now)) flag_sleep=0;
    else {
      tain_sub(&wake_time, &wake_time, &now);
      if (timeval_from_tain_relative(&timer.it_value, &wake_time)==0)
        DIE_OVERFLOW();
      flag_sleep=1;
    }
  }

  /* initialize SIGALRM action, for catching or ignoring later */
  if (sigemptyset(&action.sa_mask)!=0) DIE0(sigemptyset);
  action.sa_flags=0;

  if (flag_sleep) {
    /* schedule SIGALRM */
    timer.it_interval.tv_sec=0;
    timer.it_interval.tv_usec=0;
    if (setitimer(ITIMER_REAL, &timer, NULLP)!=0) DIE0(setitimer);

    /* catch SIGALRM */
    action.sa_handler=alarm_handler;
    if (sigaction(SIGALRM, &action, NULLP)!=0) DIE0(sigaction);

    /* sleep */
    if (sigdelset(&blockset, SIGALRM)!=0) DIE0(sigdelset);
    if (sigsuspend(&blockset)!=0 && errno!=EINTR) DIE0(sigsuspend);
  }

  /* ignore and unblock SIGALRM */
  action.sa_handler=SIG_IGN;
  if (sigaction(SIGALRM, &action, NULLP)!=0) DIE0(sigaction);
  if (sigprocmask(SIG_UNBLOCK, &alarmset, NULLP)!=0) DIE0(sigprocmask);

  argv+=2;
  exec0((char const**)argv);
  DIE1(exec, argv[0]);
}
