#include "prjlibs-include/standards.h"
#include <time.h>
#include <stdlib.h>

#include <skalibs/stddjb.h>
#include "prjlibs-include/constants.h"
#include "prjlibs-include/diewarn.h"
#include "runwhen.h"

extern char const* PROG;

void rw_scan(tain* stamp, char const* text) {
  unsigned int i, scale;
  uint64_t u;
  char const* x;
  switch (text[0]) {
    case '$':
      x=getenv(text+1);
      if (x==NULLP) DIE_MALFORMED("timestamp", text);
      text=x;
      i=tain_scan(text, stamp);
      break;
    case '@':
      ++text;
      i=tain_scan(text, stamp);
      break;
    case 'n':
      i=1;
      tain_now(stamp);
      break;
    case 'd':
      ++text;
      i=uint64_scan(text, &u);
      scale=1;
      switch (text[i]) {
        case 'W': scale*= 7; /* falls through */
        case 'd': scale*=24; /* falls through */
        case 'H': scale*=60; /* falls through */
        case 'M': scale*=60; /* falls through */
        case 'S': break;
        default: DIE_MALFORMED("timestamp", text);
      }
      ++i;
      if (u*scale/scale!=u) DIE_OVERFLOW();
      u*=scale;
      *stamp=tain_zero;
      tai_u64(tain_secp(stamp), u);
      break;
    case 't':
      ++text;
      i=uint64_scan(text, &u);
      *stamp=tain_zero;
      tai_unix(tain_secp(stamp), u);
      break;
    case 'i': {
      localtmn loc;
      short dummy;
      ++text;
      i=localtmn_scan(text, &loc);
      if (!tain_from_localtmn(stamp, &loc))
        DIE1X(100, "timestamp out of range");
      if (text[i]=='-' || text[i]=='+') {
        unsigned int j=short_scan(text+i, &dummy);
        i+=j;
        if (i<j) DIE_OVERFLOW();
      }
      break;
    }
    default:
      DIE_MALFORMED("timestamp", text);
  }
  if (i==0 || text[i]!='\0') DIE_MALFORMED("timestamp", text);
}
